#
# spec file for package zookeeper
#
# Copyright (c) 2014 SUSE LINUX Products GmbH, Nuernberg, Germany.
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via http://bugs.opensuse.org/
#

Name:           zookeeper
Version:        3.4.13
Release:        12.1
License:        Apache-2.0
Summary:        A high-performance coordination service for distributed applications
Group:          Development/Libraries/Java
Url:            http://zookeeper.apache.org/
Vendor:         Apache Software Foundation
Source0:        http://www-eu.apache.org/dist/zookeeper/zookeeper-%{version}/zookeeper-%{version}.tar.gz
Source1:        %{name}.service
Source2:        zookeeper-wrapper
Source3:        zookeeper.logrotate
Source4:        README.SUSE
Source5:        build.sh
Source6:        zkCleanTRX.sh
Patch0:         ftbfs-with-gcc-8.patch
BuildRequires:  cppunit-devel
BuildRequires:  java-devel
BuildRequires:  %{name}-kit == %{version}
%if 0%{?suse_version} >= 1210
BuildRequires:  systemd-rpm-macros
%endif
BuildRoot:      %{_tmppath}/%{name}-%{version}-%{release}-build
%{?systemd_requires}

%description
ZooKeeper is a centralized service for maintaining configuration
information, naming, providing distributed synchronization, and
providing group services. All of these kinds of services are used in
some form or another by distributed applications. Each time they are
implemented there is a lot of work that goes into fixing the bugs and
race conditions that are inevitable. Because of the difficulty of
implementing these kinds of services, applications initially usually
skimp on them ,which make them brittle in the presence of change and
difficult to manage. Even when done correctly, different
implementations of these services lead to management complexity when
the applications are deployed. This package contains the server-side
components of ZooKeeper.

%package server
Summary: Zookeeper server
Requires: java-1_8_0-openjdk
Requires: logrotate
BuildArch: noarch

%description server
This package contains the ZooKeeper server.


%package client
Summary: Zookeeper CLI clients
Requires: libzookeeper2 = %{version}

%description client
This package contains the single-threaded (zkcli_st) and multi-threaded
(zkcli_mt) command line clients for Zookeeper.


%package -n libzookeeper2
Summary: Zookeeper C client library

%description -n libzookeeper2
This package contains C client libraries for ZooKeeper.


%package -n libzookeeper2-devel
Summary: Development files for the ZooKeeper C client library
Requires: libzookeeper2 = %{version}

%description -n libzookeeper2-devel
This package contains the ZooKeeper client library's C headers and objects for
static linking.


%prep
%setup -q
%patch0 -p1
cp -Rf %{_datadir}/tetra ../kit
cp -f %{SOURCE5} ../

echo -e '\n' >> bin/zkEnv.sh

cat >> bin/zkEnv.sh << 'EOF'

for i in $(find $ZKLIB -name '*.jar')
  do
  CLASSPATH=$CLASSPATH:$i
  done
EOF

# Set log files
sed -i -e 's#^zookeeper.log.dir=.*#zookeeper.log.dir=%{_localstatedir}/log/%{name}#' \
       -e 's#^zookeeper.tracelog.dir=.*#zookeeper.tracelog.dir=%{_localstatedir}/log/%{name}#' \
           conf/log4j.properties

# Switch logging to file
echo 'log4j.rootLogger=INFO, ROLLINGFILE' >> conf/log4j.properties

sed -i 's#^dataDir=.*#dataDir=%{_localstatedir}/lib/%{name}/data#' conf/zoo_sample.cfg
sed -i -e 's#__CONFDIR__#%{_sysconfdir}/%{name}#' \
       -e 's#__LOGDIR__#%{_localstatedir}/log/%{name}#' \
       -e 's#__BINDIR__#%{_libexecdir}/%{name}#' \
       -e 's#__ZKLIB__#%{_javadir}/%{name}#' \
       %{SOURCE2}

sed -e 's#__CONFDIR__#%{_sysconfdir}/%{name}#g' \
    -e 's#__LIBEXECDIR__#%{_libexecdir}#g' \
    -e 's#__BINDIR__#%{_bindir}#g' \
    -e 's#__STATEDIR__#%{_localstatedir}/lib/%{name}#g' \
    -e 's#__LOGDIR__#%{_localstatedir}/log/%{name}#g' \
    %{SOURCE4} > README.SUSE

%build
sh ../build.sh
pushd src/c
%configure
make
popd

%install
pushd src/c
%make_install
popd

# Rename clients

for i in cli_mt cli_st load_gen
  do
  mv %{buildroot}%{_bindir}/$i %{buildroot}%{_bindir}/zk${i}
  done

# Precompiled Java libraries and dependencies
mkdir -p %{buildroot}%{_javadir}
mkdir -p %{buildroot}%{_javadir}/%{name}
cp -p build/%{name}-%{version}.jar %{buildroot}%{_javadir}/%{name}/%{name}-%{version}.jar
cp -r lib/* %{buildroot}%{_javadir}/%{name}
mkdir -p %{buildroot}%{_libexecdir}/%{name}

# Scripts

# zookeeper-wrapper:
install -p -m 755 %{SOURCE2} %{buildroot}/%{_bindir}/

# NOTE(trebskit) copy zkCleanTRX.sh to bin dir of zookeeper to allow loop below pick it up
install -p -m 755 %{SOURCE6} bin/zkCleanTRX.sh

for script in zkCleanup.sh zkCli.sh zkEnv.sh zkServer.sh zkCleanTRX.sh
  do
  # Run all executables through zookeeper-wrapper to supply SUSE specific
  # environment variables.
  ln -sr %{buildroot}%{_bindir}/%{name}-wrapper %{buildroot}%{_bindir}/$script
  install -p -m 755 bin/$script %{buildroot}%{_libexecdir}/%{name}/
  done

mkdir -p %{buildroot}%{_sysconfdir}/zookeeper
install -p -m 0640 conf/log4j.properties %{buildroot}%{_sysconfdir}/zookeeper
mkdir -p %{buildroot}%{_docdir}/zookeeper
install -p -m 0640 conf/zoo_sample.cfg %{buildroot}%{_sysconfdir}/zookeeper/zoo.cfg
touch %{buildroot}%{_sysconfdir}/zookeeper/zoo.cfg

mkdir -p %{buildroot}%{_localstatedir}/lib/zookeeper/data
echo 1 > %{buildroot}%{_localstatedir}/lib/zookeeper/data/myid

mkdir -p %{buildroot}%{_localstatedir}/log/zookeeper

install -D -m 644 %{S:1} %{buildroot}%{_unitdir}/%{name}.service
mkdir -p %{buildroot}%{_sbindir}
ln -s /usr/sbin/service %{buildroot}%{_sbindir}/rc%{name}

install -p -D -m 644 %{SOURCE3} %{buildroot}%{_sysconfdir}/logrotate.d/%{name}-server

%pre server
groupadd --system zookeeper 2>/dev/null || :
useradd -g zookeeper -r -d %{_localstatedir}/lib/%{name} -s /bin/false \
	-c "ZooKeeper service account" %{name} 2>/dev/null || :
%service_add_pre %{name}.service

%post -n libzookeeper2
/sbin/ldconfig > /dev/null 2>&1

%postun -n libzookeeper2
/sbin/ldconfig > /dev/null 2>&1

%post server
%service_add_post %{name}.service

%preun server
%service_del_preun %{name}.service

%postun server
%service_del_postun %{name}.service

%files server
%defattr(-,root,root,-)
%license LICENSE.txt
%doc README.SUSE
%config %{_sysconfdir}/logrotate.d/%{name}-server
%{_bindir}/zkCleanup.sh
%{_bindir}/zkCli.sh
%{_bindir}/zkEnv.sh
%{_bindir}/zkServer.sh
%{_bindir}/zookeeper-wrapper
%{_bindir}/zkCleanTRX.sh
%{_libexecdir}/%{name}
%{_javadir}/%{name}/
%attr(0750,zookeeper,zookeeper) %dir /var/lib/zookeeper
%attr(0750,zookeeper,zookeeper) %dir /var/lib/zookeeper/data
%attr(0640,zookeeper,zookeeper) %config(noreplace) /var/lib/zookeeper/data/myid
%attr(0755,zookeeper,zookeeper) %dir /var/log/zookeeper
%attr(0755,root,root) %dir %{_sysconfdir}/zookeeper
%attr(0644,root,root) %config(noreplace) %{_sysconfdir}/zookeeper/zoo.cfg
%attr(0644,root,root) %config(noreplace) %{_sysconfdir}/zookeeper/log4j.properties
%{_unitdir}/%{name}.service
%{_sbindir}/rc%{name}

%files client
%defattr(-,root,root,-)
%license LICENSE.txt
%doc README.SUSE
%{_bindir}/zkcli_mt
%{_bindir}/zkcli_st
%{_bindir}/zkload_gen

%files -n libzookeeper2
%defattr(-,root,root,-)
%license LICENSE.txt
%doc README.SUSE
%{_libdir}/libzookeeper_mt.so.2
%{_libdir}/libzookeeper_mt.so.2.0.0
%{_libdir}/libzookeeper_st.so.2
%{_libdir}/libzookeeper_st.so.2.0.0

%files -n libzookeeper2-devel
%defattr(-,root,root,-)
%license LICENSE.txt
%doc README.SUSE
%{_includedir}/zookeeper
%{_libdir}/libzookeeper_mt.a
%{_libdir}/libzookeeper_mt.la
%{_libdir}/libzookeeper_mt.so
%{_libdir}/libzookeeper_st.a
%{_libdir}/libzookeeper_st.la
%{_libdir}/libzookeeper_st.so

%changelog
* Mon Jan  7 2019 tbechtold@suse.com
- update to 3.4.13:
  - Backport ZOOKEEPER-2939 Deal with maxbuffer as it relates
    to proposals
  - Zookeeper Client should re-resolve hosts when connection
    attempts fail
  - Upgrade OWASP Dependency Check to 3.2.1
  - ignore accepted epoch and LEADERINFO ack from observers when
    a newly elected leader computes new epoch
  - NPE triggered if server receives a vote for a server id not
    in their voting view
  - .ignore file prevents adding
    src/java/main/org/apache/jute/compiler/generated dir to git repo
  - Potential NPE in ReferenceCountedACLCache#deserialize
  - Accidently removed public API of FileTxnLog.setPreallocSize()
  - TxnLogToolkit uses Scanner badly
  - Typo in error message, affects log analysis
  - Tool required to recover log and snapshot entries with CRC
    errors
  - Fix unit test: testDataDirAndDataLogDir should not use
    hardcode test folders
  - Add a metric to track number of slow fsyncs
  - QuorumKerberosHostBasedAuthTest fails on Linux box: Unable to
    parse:includedir /etc/krb5.conf.d/
  - SessionTest is using Thread deprecated API.
  - Enable Clover code coverage report
  - Add C client code coverage tests
  - CRC check failed when preAllocSize smaller than node data
  - Update documentation source for ZOOKEEPER-2574
  - Flaky test: org.apache.zookeeper.server.quorum.FLEBackward \
    ElectionRoundTest.testBackwardElectionRound
  - Data inconsistency issue due to retain database in leader
    election
  - very poor choice of logging if client fails to connect to server
  - The comment of the variable matchSyncs in class CommitProcessor
    has a mistake.
  - Flaky Test: org.apache.zookeeper.test.LoadFromLogTest. \
    testRestoreWithTransactionErrors
  - WriteLock recipe: incorrect znode ordering when the sessionId
    is part of the znode name
  - Duplicate Keys in log4j.properties config files
  - Specify correct overflow value
  - Failing c unit tests on apache jenkins
  - zkServer.cmd does not start when JAVA_HOME ends with a \
  - Flaky Test: testNoLogBeforeLeaderEstablishment
  - The dataDir and dataLogDir are used opposingly
  - Fix testElectionFraud Flakyness
  - fix potential null pointer exception when deleting node
  - The eclipse build target fails due to protocol
    redirection: http->https
  - Add keys for the Zxid from the stat command to check_zookeeper.py
  - Upgrade third party libraries to address vulnerabilities
  - The function queueEmpty() in FastLeaderElection.Messenger
    is not used, should be removed.
  - Add check to validate dataDir and dataLogDir parameters
    at startup
  - Change log level for "ZKShutdownHandler is not registered"
    error message
  - Fix "Unexpected bean exists!" issue in WatcherTests
  - Cleanup findbug warnings in branch-3.4:
  * Correctness Warnings
  * Disable Internationalization Warnings
  * Malicious code vulnerability Warnings
  * Performance Warnings
  * Dodgy code Warnings
  * Experimental Warnings
  - Set up Apache Jenkins job that runs the flaky test analyzer
    script
  - Multithreaded correctness Warnings
  - ZOOKEEPER-2355 fix for branch-3.4
  - Windows: fetch_and_add not 64bit-compatible, may not be correct
  - Update documentation for snapCount
  - Ephemeral node is never deleted if follower fails while
    reading the proposal packet
  - Port ZOOKEEPER-1576 to branch3.4
  - recreateSocketAddresses may recreate the unreachable IP address
  - Flaky Test: org.apache.zookeeper.test.ReadOnlyModeTest. \
    testSessionEstablishment
  - Clean up findbug warnings in branch-3.4
  - Port ZOOKEEPER-2737 to branch-3.4
  - Netty connection leaks JMX connection bean upon connection
    close in certain race conditions.
  - Typo: transasction --> transaction
  - Flaky test: org.apache.zookeeper.server.quorum. \
    QuorumCnxManagerTest.testNoAuthLearnerConnectToAuth \
    RequiredServerWithHigherSid
  - Ephemeral znode will not be removed when sesstion timeout, if
    the system time of ZooKeeper node changes unexpectedly.
  - ZK Client not able to connect with Xid out of order error
  - There is a typo in zk.py which prevents from using/compiling it.
  - follower disconnects and cannot reconnect
  - Server inappropriately throttles connections under load before
    SASL completes
  - Flaky test: org.apache.zookeeper.test.ClientTest. \
    testNonExistingOpCode
  - Flaky test: org.apache.zookeeper.test.ReadOnlyModeTest. \
    testConnectionEvents
  - Unnecessary stack-trace in server when the client disconnect
    unexpectedly
  - PurgeTxnLog#validateAndGetFile: return tag has no arguments.
  - Improve the ZooKeeper#setACL java doc
  - ZooKeeper public include files leak porting changes
  - CMake build doesn't support OS X
  - Main-Class JAR manifest attribute is incorrect
  - Windows Debug builds don't link with `/MTd`
  - Local automatic variable is left uninitialized and then freed.
  - Don't include `config.h` in `zookeeper.h`
  - The OWASP dependency check jar should not be included in the
    default classpath
  - quorum.auth.MiniKdcTest.testKerberosLogin failing with NPE
    on java 9
  - Create ant task to generate ivy dependency reports
  - compiler warning using java 9
  - Operations to server will be timed-out while thousands of
    sessions expired same time
  - TCP keepalive for leader election connections
  - The define of MAX_CONNECTION_ATTEMPTS in QuorumCnxManager.java
    seems useless, should it be removed?
  - ZooKeeperSaslClient#respondToServer should log exception
    message of SaslException
  - Add script to run a java api compatibility tool
  - Improve the efficiency of AtomicFileOutputStream
  - Rename README.txt to README.md
  - define dependency versions in build.xml to be easily overridden
    in build.properties
  - Please add instructions for running the tutorial
  - Add ant task for running OWASP dependency report
  - Flaky Test: org.apache.zookeeper.test.WatcherTest.
- Drop ZOOKEEPER-705.patch . Applied upstream
- Drop ZOOKEEPER-770.patch . Applied upstream
- Add ftbfs-with-gcc-8.patch . This fixes the build with gcc8
- Use %%license macro
* Mon Jul 10 2017 johannes.grassler@suse.com
- Fixed off-by-one in zkCleanTRX.sh and made output more useful
  (bsc#1048688, FATE#323204)
* Mon Jul  3 2017 johannes.grassler@suse.com
- Fixed ExecStartPre statment in service file
* Mon Jun 26 2017 tomasz.trebski@ts.fujitsu.com
- added zkCleanTRX.sh to clean up 0 length transaction logs
* Mon May 29 2017 johannes.grassler@suse.com
- Update to to zookeeper-3.4.10 (bsc#1040519)
  * Fixes CVE-2017-5637
- Remove Changes.txt (missing as of 3.4.10)
* Wed Mar 29 2017 dmueller@suse.com
- add ZOOKEEPER-705.patch, ZOOKEEPER-770.patch
* Tue Feb 14 2017 johannes.grassler@suse.com
- Switched from using binary jars to Tetra based build from source
* Fri Jan 13 2017 johannes.grassler@suse.com
- Added README.SUSE
- Replaced copies of zookeeper-wrapper by symlinks
- Created subpackages zookeeper-client, zookeeper-server,
  libzookeeper and libzookeeper-devel
- Reduced log level to INFO
* Thu Jan 12 2017 johannes.grassler@suse.com
- Copy sample configration file to /etc/zookeeper/zoo.cfg
- Configure log4j for logging to /var/log
- Fixed class path handling
- Moved all Zookeeper jars to /usr/share/zookeeper and
  removed unneccessary symlinks
- Added logrotate configuration
- Set a default server ID of 1
- Got rid of unneccessary ExecStop directive
* Wed Jan 11 2017 johannes.grassler@suse.com
- Bumped version to 3.4.9
- Updated URL to point to www-eu.apache.org
- Substituted zkEnv.patch by environment wrapper
- Substituted zkServer.patch by appending proper CLASSPATH to
  zkEnv.sh
- Substituted some more RPM macros for static paths where
  appropriate
- Moved zoo_sample.cfg to docdir
* Thu Aug 13 2015 aplanas@suse.com
- Initial package
